/* cwpcm.c - text to International Morse Code converter 
Copyright (C) 2001 Randall S. Bohn

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
/* cwmm - output XML morse code compatible with MorseMail. See
http://www.seanet.com/~harrypy/MorseMail/ for details. */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "morse.h"
/*#include "pcm.h"*/

/* to match MM we'll call a 1wpm dit=1188 msec mark, 1188 msec space */
#define tbase 1188
#define csegment (tbase/cwpm)
#define wsegment (tbase/wwpm)

/* rates (character, word) for various speeds (slow|med|fast|extra) */
#define CSLOW 12
#define WSLOW 5
#define CMED 18
#define WMED 12
#define CFAST 18
#define WFAST 18
#define CEXTRA 20
#define WEXTRA 20

#define MMVERSION "2001.05.27"
#define MMREADER "<readerURL>\nhttp://www.seanet.com/~harrypy/MorseMail/\n</readerURL>\n"

/* program settings */
/* character rate */
int cwpm = CFAST;
/* word rate */
int wwpm = WFAST;
/* storage for space time (hi) */
int spaceAccumulator = 0;

/* slow down the wpm rate if we use a faster character rate */
int adjustWpm(int c_rate, int w_rate) {
 /* if the word rate is faster than the char rate go with the char rate */
 /* if they are the same that is fine, no ajustment needed */
 if (w_rate >= c_rate) return c_rate;

 /* slow down a bit, the bigger the difference the more we slow down. */
 w_rate=w_rate - 2*(c_rate - w_rate)/w_rate;

 /* let's not be silly now */
 if (w_rate >= 1) return w_rate;
 else return 1;
}

/* morsemail xml support */
void openMorseMail(FILE *out) {
 fputs("<MorseMail>", out);
 fputs(MMVERSION, out);
 fputs("\n", out); /* not mac friendly... */
}

void closeMorseMail(FILE *out) {
 fputs("</MorseMail>\n", out);
}

void addReaderInfo(FILE *out) {
 fputs(MMREADER, out);
}

/* morsemail mark and space */
/* have to save up spaces and just put out one,
	morsemail doesn't like having two spaces in a row.
	So, accumulate spaces, put the total time out before
	each mark. Be sure to call finish(out) to emit the
	final space (you will have final space!) */
void mark(int duration, FILE *out) {
 if (spaceAccumulator > 0) {
  fprintf(out, "%+d", -spaceAccumulator);
  spaceAccumulator = 0;
 }
 fprintf(out, "%+d", duration);
}
void space(int duration, FILE *out) {
 spaceAccumulator += duration;
}

void setup() {
 spaceAccumulator = 0;
}
void finish(FILE *out) {
 if (spaceAccumulator > 0) {
  fprintf(out, "%+d", -spaceAccumulator);
 }
 fputs("\n", out);
 fflush(out);
}

/* morse-speak: */
void dit(FILE *out) {
 mark(csegment, out);
 space(csegment, out);
}
void dah(FILE *out) {
 mark(csegment * 3, out);
 space(csegment, out);
}
void err(FILE *out) {
}

void cspace(FILE *out) {
 space(wsegment*3, out);
}
void wspace(FILE *out) {
 cspace(out);
}


int getInt(char *s, int low, int high) {
	int ival;
	char *temp = malloc(6);
	strncpy(temp, s, 5);
	if (strlen(s) > 5) *(temp+5) = '\0';
	ival = atoi(temp);
	free(temp);
	if (ival < low) return low;
	if (ival > high) return high;
	return ival;
}

int main(int argc, char **argv) {
 int ch, x;
 FILE *out = stdout;


 /* decode startup options */
 for (x = 0; x < argc; x++) {
  if (strncmp(argv[x], "-ss", 3)==0) {
   cwpm = CSLOW;
   wwpm = WSLOW;
  }
  if (strncmp(argv[x], "-sm", 3)==0) {
   cwpm = CMED;
   wwpm = WMED;
  }
  if (strncmp(argv[x], "-sf", 3)==0) {
   cwpm = CFAST;
   wwpm = WFAST;
  }
  if (strncmp(argv[x], "-sx", 3)==0) {
   cwpm = CEXTRA;
   wwpm = WEXTRA;
  }

 }
 wwpm=adjustWpm(cwpm, wwpm);

 openMorseMail(out);
 setup();
 while ((ch = fgetc(stdin)) != EOF) {
  if (isspace(ch)) wspace(out);
  else genMorse(tolower(ch), out);
 }
 finish(out);
 closeMorseMail(out);
 addReaderInfo(out);
 fflush(out);
 return 0;
}
