/* cwpcm.c - text to International Morse Code converter 
Copyright (C) 2001 Randall S. Bohn

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "morse.h"
#include "pcm.h"

/* a dit at 1 wpm is 1177 msec sound followed by 1177 msec silence */
/* a dah at 1 wpm is 3531 msec (3*1177) sound, 1177 msec silence */
/* 1177 msec * 8 samples per msec */
#define tbase 9416
#define csegment (tbase/cwpm)
#define wsegment (tbase/wwpm)
/* 900 Hz */
#define PITCH 900

/* rates (character, word) for various speeds (slow|med|fast|extra) */
#define CSLOW 12
#define WSLOW 5
#define CMED 18
#define WMED 12
#define CFAST 18
#define WFAST 18
#define CEXTRA 20
#define WEXTRA 20

/* program settings */
int pcm = 1;
int hvox = 0;
/* audio character rate */
int cwpm = CFAST;
/* audio word rate */
int wwpm = WFAST;

/* slow down the wpm rate if we use a faster character rate */
int adjustWpm(int c_rate, int w_rate) {
 /* if the word rate is faster than the char rate go with the char rate */
 /* if they are the same that is fine, no ajustment needed */
 if (w_rate >= c_rate) return c_rate;

 /* slow down a bit, the bigger the difference the more we slow down. */
 w_rate=w_rate - 2*(c_rate - w_rate)/w_rate;

 /* let's not be silly now */
 if (w_rate >= 1) return w_rate;
 else return 1;
}

/* morse-speak: */
void dit(FILE *out) {
 mark(hvox, csegment, out);
 space(hvox, csegment, out);
}
void dah(FILE *out) {
 mark(hvox, csegment * 3, out);
 space(hvox, csegment, out);
}
void err(FILE *out) {
}

void cspace(FILE *out) {
 space(hvox, wsegment*3, out);
}
void wspace(FILE *out) {
 cspace(out);
 fflush(out);
}

void setupVoice(int hz, int amp) {
 hvox = voiceFactory(hz, amp, 128, 8000);
}

int getInt(char *s, int low, int high) {
	int ival;
	char *temp = malloc(6);
	strncpy(temp, s, 5);
	if (strlen(s) > 5) *(temp+5) = '\0';
	ival = atoi(temp);
	free(temp);
	if (ival < low) return low;
	if (ival > high) return high;
	return ival;
}

int main(int argc, char **argv) {
 int ch, x, pitch;
 FILE *out = stdout;

 pitch = PITCH;
 /* decode startup options */
 for (x = 0; x < argc; x++) {
  if (strncmp(argv[x], "-ss", 3)==0) {
   cwpm = CSLOW;
   wwpm = WSLOW;
  }
  if (strncmp(argv[x], "-sm", 3)==0) {
   cwpm = CMED;
   wwpm = WMED;
  }
  if (strncmp(argv[x], "-sf", 3)==0) {
   cwpm = CFAST;
   wwpm = WFAST;
  }
  if (strncmp(argv[x], "-sx", 3)==0) {
   cwpm = CEXTRA;
   wwpm = WEXTRA;
  }
  if (strncmp(argv[x], "-f:", 3)==0) {
   pitch = getInt(argv[x]+3, 40, 3200);
  }
 }
 wwpm=adjustWpm(cwpm, wwpm);
 setupVoice(pitch, 90);

 while ((ch = fgetc(stdin)) != EOF) {
  if (isspace(ch)) wspace(out);
  else genMorse(tolower(ch), out);
 }
 return 0;
}
